from datetime import datetime, timezone

from flaschengeist.models.user import User

from flaschengeist.database import db
from flaschengeist.models import ModelSerializeMixin, UtcDateTime


class Transaction(db.Model, ModelSerializeMixin):
    __tablename__ = "balance_transaction"
    id: int = db.Column("id", db.Integer, primary_key=True)
    time: datetime = db.Column(UtcDateTime, nullable=False, default=datetime.now(tz=timezone.utc))
    amount: float = db.Column(db.Numeric(precision=5, scale=2, asdecimal=False), nullable=False)

    # Dummy properties used for JSON serialization (userid instead of full user)
    sender_id: str = ""
    receiver_id: str = ""
    author_id: str = ""

    # Protected foreign key properties
    _receiver_id = db.Column("receiver_id", db.Integer, db.ForeignKey("user.id"))
    _sender_id = db.Column("sender_id", db.Integer, db.ForeignKey("user.id"))
    _author_id = db.Column("author_id", db.Integer, db.ForeignKey("user.id"), nullable=False)

    # Not exported relationships just in backend only
    sender_: User = db.relationship("User", foreign_keys=[_sender_id])
    receiver_: User = db.relationship("User", foreign_keys=[_receiver_id])
    author_: User = db.relationship("User", foreign_keys=[_author_id])

    @property
    def sender_id(self):
        return self.sender_.userid

    @property
    def receiver_id(self):
        return self.receiver_.userid

    @property
    def author_id(self):
        return self.author_.userid
